*! version 5.4 JACS/KT g-estimation, 2 August 2001
program define stgest, rclass
  version 7.0
  syntax varlist, visit(varname)[ lasttime(string) range(numlist) /*
    */ step(real 0) tol(int 3) lagconf(varlist) baseconf(varlist) /*
    */ firstvis(string) pnotcens(varname) idcens(varname) /*
    */ saveres(string) detail replace round(int 6) level(int $S_level) *]
  tokenize `varlist'
  local causvar `1'
  local cmd "logistic"
  capture assert `causvar'==0|`causvar'==1
    if _rc~=0 {
      display as error "Causal variable must be 0/1 "
      exit 999
    }

  local zcrit = invnorm(1-(100-`level')/200)
  local zcri = round(`zcrit',.01)
  local round=10^(-`round')

  if "`firstvis'"=="" {
    quietly summ `visit'
    local firstvis=r(min)
  }

  mac shift
  local conf "`*'"
  display as text "confounders: `conf'"
  display as text "causvar: `causvar'"
  display as text "visit: `visit'"

  local idvar: char _dta[st_id]
  confirm variable `idvar'
  confirm variable `visit'
  if "`lasttime'"~="" {
    confirm variable `lasttime'
  }
  if "`pnotcens'"~="" {
    confirm variable `pnotcens'
    capture confirm variable `idcens'
    if _rc~=0 {
      display as error "Indicator variable for ids censored must be supplied if censoring present"
      exit 999
    }
    capture assert `idcens'==0|`idcens'==1|`idcens'==.
    if _rc~=0 {
      display as error "`idcens' should take values 0 or 1 only"
      exit 999
    }
  }
  if "`saveres'"=="" {
    local saveres "gestres"
    local replace "replace"
  }
  if "`detail'"=="" {
    local qui "quietly"
  }
  if "`range'"=="" {
    local range "-5 5"
  }
  local rnum: word count `range'
  display as text "Range: `range', rnum: `rnum'"
  capture assert `rnum'==2
  if _rc~=0 {
    display as error "Range should contain start and end of search region"
  }
  local psimin: word 1 of `range' 
  local psimax: word 2 of `range'
  if `step'==0 {local search interval bisection}
  else {local search grid search}
  noi di in green "Search method: `search'"

  tempvar intpsi u delt delt1 touse maxvis btemp cofpsi testdiff censid
  tempname memhold gest pval z

  capture drop B*
  if "`baseconf'"~="" {
    quietly summ `visit'
    local basevis=r(min)
    unab tmplst: `baseconf'
    tokenize `tmplst'
    while "`1'"~="" {
      local newname=substr("`1'",1,14)
      local newname="B`newname'"
      capture drop `btemp'
      quietly gen `btemp'=`1' if `visit'==`basevis'
      egen `newname'=max(`btemp'), by(`idvar')
      mac shift
    }
    display as text _n "Baseline confounders"
    describe B*
    local B "B*"
  }
  sort `idvar' `visit'
  capture drop L*
  if "`lagconf'"~="" {
    unab tmplst: `lagconf'
    tokenize `tmplst'
    while "`1'"~="" {
      local newname=substr("`1'",1,14)
      local newname="L`newname'"
      quietly by `idvar': gen `newname'=`1'[_n-1]
      mac shift
    }
    display as text _n "Lagged confounders"
    describe L*
    local L "L*"
  }

  gen byte `touse'=1
  markout `touse' `conf' `B' `L'
  quietly count if `touse' & `visit'>=`firstvis'
  local totobs=r(N)
  egen `maxvis'=max(`visit'), by(`idvar')
  postfile `memhold' psi pval z error using `saveres', `replace'

  * SET UP LOOP
  if (`psimin' >= `psimax') {
     di in red "psimin must be < psimax"
     exit 498
     }
  local stop 0
  local psi = `psimin'
  local estlo = .
  local esthi = .
  local lowlo = .
  local lowhi = .
  local upplo = .
  local upphi = .

  ****** START LOOP ******
  while `stop' ~= 1 {

    sort `idvar' `visit'
    * contribution to psi in each interval
    capture drop `intpsi' `u' `delt'
    capture drop `cofpsi' `testdiff'
    capture drop `censid'

    quietly gen `intpsi'=(_t-_t0)*exp(`causvar'*`psi') if `visit'>=`firstvis'

    * Generate psi by summing over intervals
    quietly egen `u'=sum(`intpsi') if `visit'>=`firstvis', by(`idvar')
 
    if "`lasttime'"=="" {
      * no need to create indicator variable if there is no censoring
      * (note - this option not tested yet - 22 June 2001)
      gen `delt'=`u'
    }

    if "`lasttime'"~="" {
      * generate indicator variable to allow for censoring
      quietly gen `cofpsi'=`lasttime' if `psi'>=0  & `visit'>=`firstvis'
      quietly replace `cofpsi'=`lasttime'*exp(`psi') if `psi'<0 & `visit'>=`firstvis'
      quietly gen float `testdiff'=abs(float(`u')-float(`cofpsi'))
      quietly gen byte `delt'=1 if (float(`u')<float(`cofpsi')&`testdiff'>float(`round')) /*
       */ & `visit'>=`firstvis'
      quietly replace `delt'=0 if (float(`u')>=float(`cofpsi')|`testdiff'<=float(`round')) /*
       */ & `visit'>=`firstvis'

      quietly egen `censid'=max(_t), by(`idvar')
      quietly replace `censid'=cond((abs(`censid'-`lasttime')<0.001),1,0)
      capture assert ~(`delt'==1&`censid'==1)
      if _rc~=0 {
        display _n as error "Rounding problem with date of end of follow up."
        display as error "Indicator variable delta should not be 1 if an id was censored."
        display as error "The difference between the two rightmost variables in the list"
        display as error "below should be less than 0.001. Try a lower number in the round()
        display as error "option, or change the scale on which time is measured."
        sort id visit
        list id visit _t0 _t `lasttime' `u' `cofpsi' if `delt'==1&`censid'==1 
        exit 999
      }
    }
    if "`detail'"=="" {
      display as result %4.2f `psi' " " _c
    }
    else {
      display _n %4.2f "Iterating: psi=`psi'"
    }
    if "`pnotcens'"=="" {
      `qui' `cmd' `causvar' `delt' `conf' `B' `L' if `visit'>=`firstvis'
    }
    else if "`pnotcens'"~="" {
      quietly replace `delt'=`delt'/`pnotcens' if `idcens'==0&`visit'>=`firstvis'
      quietly replace `delt'=0 if `idcens'==1&`visit'>=`firstvis'
      `qui' `cmd' `causvar' `delt' `conf' `B' `L' if `visit'>=`firstvis', /*
       */ robust cluster(`idvar')
    }

    quietly count if e(sample)
    scalar error=0
    capture assert r(N)==`totobs'       
    if _rc>0 {
      scalar error=1
    }
    capture {
      scalar `gest'=`psi'
      scalar `z'=(_coef[`delt'])/(_se[`delt'])
      scalar `pval'=2*min((1-normprob(`z')),normprob(`z'))
      
      post `memhold' (`gest') (`pval') (`z') (error)
    }

    * update results
    if `z'<0   {local esthi = min(`esthi',`psi')}
          else {local estlo = max(`estlo',`psi')}
    if `z'<-`zcrit' {local upphi = min(`upphi',`psi')}
          else      {local upplo = max(`upplo',`psi')}
    if `z'<`zcrit' {local lowhi = min(`lowhi',`psi')}
          else     {local lowlo = max(`lowlo',`psi')}
    if "`detail'"=="detail" {noi di _n "psi=`psi' Z=" %5.2f `z' /*
       */ " esthi=`esthi' estlo=`estlo' upphi=`upphi' upplo=`upplo'" /*
       */ " lowhi=`lowhi' lowlo=`lowlo' stop=`stop'"}

    * CHOOSE NEXT PSI
    if `step' == 0 {
      * interval bisection
      local estOK = (`esthi'-`estlo')<10^(-`tol') | `esthi'==. | `estlo'==.
      local hiOK  = (`upphi'-`upplo')<10^(-`tol') | `upphi'==. | `upplo'==.
      local lowOK = (`lowhi'-`lowlo')<10^(-`tol') | `lowhi'==. | `lowlo'==.
      if "`detail'"=="detail" {
        noi di "estOK=`estOK', lowOK=`lowOK', hiOK=`hiOK'"
      }

      if `psi' == `psimin' {local psi = `psimax'}
      else if ~`estOK' {
         if "`detail'"=="detail" {noi di _n "improving estimate"}
         local psi  = (`esthi'+`estlo')/2
      }
      else if ~`hiOK' & "`ci'"~="noci" {
         if "`detail'"=="detail" {noi di _n "improving upper CL"}
         local psi  = (`upphi'+`upplo')/2
      }
      else if ~`lowOK' & "`ci'"~="noci" {
         if "`detail'"=="detail" {noi di _n "improving lower CL"}
         local psi  = (`lowhi'+`lowlo')/2
      }
      else {
         if "`detail'"=="detail" {noi di "FINISH!!!"}
         local stop 1
      }
    }                    /* END OF INTERVAL BISECTION LOOP */
    else {
      * grid search
      local psi = `psi' + `step'
      local stop = `psi'>`psimax'
    }
  }

  postclose `memhold'
  display _n(2)
  
  preserve
  display as text "savres: `saveres'"
  quietly use `saveres', clear
  label data "Results of G-estimation with causal variable `causvar'"
  quietly {
    count if error~=0
    if r(N)==0 {
      drop error
    }
    sort psi
    save, replace
  }
  tempvar testp
  sort psi
  list
  quietly summ pval
  quietly summ psi if pval==r(max)
  quietly local gest=(r(max)+r(min))/2
  quietly local rrcaus=exp(-`gest')

  gen `testp'=abs(pval-0.05)
  quietly summ `testp' if z<-1
  quietly summ psi if `testp'==r(min) & z<-1
  local gestu=(r(max)+r(min))/2

  quietly replace `testp'=abs(pval-0.05)
  quietly summ `testp' if z>1
  quietly summ psi if `testp'==r(min) & z>1
  local gestl=(r(max)+r(min))/2

  local rrcausu=exp(-`gestl')
  local rrcausl=exp(-`gestu')

  display as text _n "G estimate of psi for `causvar': " as result %5.3f `gest' " (95% CI " %5.3f `gestl' " to "%5.3f `gestu' ")"
  display _n as text "Causal survival time ratio for `causvar': " as result %5.3f `rrcaus' " (95% CI " %5.3f `rrcausl' " to "%5.3f `rrcausu' ")"

  graph z psi if z<2.5&z>-2.5, yline(-1.96, 0, 1.96) xlab ylab xline(`gestl', `gest', `gestu') `options'

  return scalar psi=`gest'
  return scalar lcipsi=`gestl'
  return scalar ucipsi=`gestu'
  return scalar trcaus=`rrcaus'
  return scalar utrcaus=`rrcausl'
  return scalar ltrcaus=`rrcausu'

  global psi=`gest'
  global lcipsi=`gestl'
  global ucipsi=`gestu'
end

